using System;
using TIS.Imaging;

/* 
 * Using the filter
 * 
 * 1.) In the main form create a variable of this class type:
 * 
 *     private InternalContrastEnhancement MyInternalContrastEnhancement;
 *
 * 2.) Instantiate the class in the form load method of the main form:
 * 
 *		MyInternalContrastEnhancement = new InternalContrastEnhancement();
 * 
 * 3.) Insert the filter in IC Imaging Control, e.g. as device filter. This can be
 *     done in the form load event too:
 * 
 *     icImagingControl1.DeviceFrameFilters.Add(  icImagingControl1.FrameFilterCreate( MyInternalContrastEnhancement ) );
 * 
 * 4.) To see the result of the filter, enable it:
 *     MyInternalContrastEnhancement.setEnable(true);
 * 
 * A form load method could look like the following code:
 * 
 *      private void Form1_Load(object sender, System.EventArgs e)
 *		{
 *			MyInternalContrastEnhancement = new InternalContrastEnhancement();
 *			icImagingControl1.DeviceFrameFilters.Add(  icImagingControl1.FrameFilterCreate( MyInternalContrastEnhancement ) );
 *			MyInternalContrastEnhancement.setEnable(true);
 *		}
 */		


namespace ContrastEnhancement3
{
	// [FilterInfo("Test1")]
	public class InternalContrastEnhancement : FrameFilterImpl
	{	
		private bool m_bEnabled = false; // This variable is used to enable and disable the
										 // contrast enhancement image processing.
		private long m_lowerBound = 50;  // The lower bound of the interval the pixel values are mapped to.
		private long m_upperBound = 250; // The upper bound of the interval the pixel values are mapped to.
		
		public void ExternalBeginParameterTransfer()
		{
			BeginParameterTransfer();
		}

		public void ExternalEndParameterTransfer()
		{
			EndParameterTransfer();
		}


		public InternalContrastEnhancement()
		{
			this.AddBoolParam( "Enable", new TIS.Imaging.FrameFilterImpl.SetBoolParam( setEnable ), new TIS.Imaging.FrameFilterImpl.GetBoolParam( getEnable ) );
		}

		/*
		 *	Enables or disables the image processing in the filter.
		 *
		 *	Only call this method in a beginParamTransfer/endParamTransfer block.
		 */
		public void	setEnable( bool bEnable )
		{
			m_bEnabled = bEnable;
		}

		/*
		 *	Get the current enabled state of the filter.
		 */
		public bool	getEnable()
		{
			return m_bEnabled;
		}

		public bool setLowerBound( long lLowerBound )
		{
			// Check boundary conditions.
			if( (lLowerBound >= 0) && (lLowerBound < 256) && (lLowerBound < m_upperBound) )
			{
				m_lowerBound = lLowerBound;
				return true;
			}				
			return false;
		}

		public long getLowerBound()
		{
			return m_lowerBound;
		}

		public bool setUpperBound( long lUpperBound )
		{
			// Check boundary conditions.
			if( (lUpperBound >= 0) && (lUpperBound < 256) && (lUpperBound > m_lowerBound) )
			{
				m_upperBound = lUpperBound;
				return true;
			}
			return false;
		}

		public long getUpperBound()
		{
			return m_upperBound;
		}

		/*
		 * This method fills the ArrayList arr with the frame types this filter
		 * accepts as input.
		 */
		public override void GetSupportedInputTypes(System.Collections.ArrayList frameTypes)
		{
			// This filter works for all color formats
			frameTypes.Add( new TIS.Imaging.FrameType( TIS.Imaging.MediaSubtypes.Y800 ) );
			frameTypes.Add( new TIS.Imaging.FrameType( TIS.Imaging.MediaSubtypes.Y16 ) );
			frameTypes.Add( new TIS.Imaging.FrameType( TIS.Imaging.MediaSubtypes.RGB24 ) );
			frameTypes.Add( new TIS.Imaging.FrameType( TIS.Imaging.MediaSubtypes.RGB32 ) );
		}


		/*
		 *	This method returns the output frame type for a given input frame type.
		 */
		public override bool GetTransformOutputTypes(FrameType inType, System.Collections.ArrayList outTypes)
		{
			outTypes.Add( inType );

			return true;
		}

		/*
		 *	This method is called to copy the image data from the src frame to the dest frame.
		 *
		 *	Depending on the value of m_bEnabled, this implementation applies a contrast enhancement or
		 *	copies the image data without modifying it.
		 */
		public override bool Transform(IFrame src, IFrame dest)
		{
			byte [] LUT = new Byte [256];
			unsafe
			{
				// Check whether the destination frame is available.
				if( dest.Ptr == null ) return false;

				// Copy the member variables to the function's stack, to protect them from being
				// overwritten by parallel calls to setThreshold() etc.
				//
				// beginParamTransfer/endParamTransfer makes sure that the values from various
				// member variables are consistent, because the user of this filter must enclose
				// writing parameter access into beginParamTransfer/endParamTransfer, too.
				BeginParameterTransfer();
				bool enabled = m_bEnabled;
				long upperBound = m_upperBound;
				long lowerBound = m_lowerBound;
				EndParameterTransfer();

				// Initialize a lookup table with values between lowerBound and upperBound.
				for( int i = 0; i < 256; ++i )
				{
					if( i <= lowerBound )
					{
						LUT[i] = 0;
					}
					else if( i >= upperBound )
					{
						LUT[i] = 255;
					}
					else
					{
						// Map the pixel values between lowerBound and upperBound to the range 0 to 255.
						LUT[i] = System.Convert.ToByte( System.Convert.ToDouble(i - lowerBound) / 
										System.Convert.ToDouble(upperBound - lowerBound) * 256.0);
					}
				}

				byte* pIn = src.Ptr;
				byte* pOut = dest.Ptr;

				// Check whether the filter is enabled.
				if( enabled )
				{
					// Map all pixel values to a range defined by the lookup table.
					int bufferSize = src.FrameType.BufferSize;
					byte* pSource = src.Ptr;
					byte* pTarget = dest.Ptr;
					byte *pEnd =  &pSource[bufferSize];
					
					while( pSource < pEnd )
					{
						*pTarget = LUT[*pSource];
						pSource++;
						pTarget++;
					}
				}
				else
				{
					// The filter  is disabled: Copy the image data without modifications.
					int bufferSize = src.FrameType.BufferSize;
					while( bufferSize-- > 0 )
					{
						*pOut++ = *pIn++;
					}
				}
			}

			return true;
		}
	}
}
